'use strict';
angular.module('myApp.controllers', [])
    .controller('MainCtrl', ['$scope', '$rootScope', '$window', '$location', function ($scope, $rootScope, $window, $location) {
        $scope.bottomLinkHeight =bottomLinkHeight();
        animationStyles();
        $scope.slide = '';
        $rootScope.back = function() {
            if($scope.pageDepth == 2){
                //$scope.slide = 'slide-right';
                $window.history.back();
            }
        }
        $rootScope.go = function(path, pageNum){
            if(pageNum < $scope.currentPage){
                $scope.slide = 'slide-right';
            }
            else{
                $scope.slide = 'slide-left';
            }
            $scope.currentPage = pageNum;
            $location.url(path);
        }
        $scope.pageLoading = true;
        $rootScope.resolvePageLoad = function(){
            setIscroll();
            $scope.pageLoading = false;
        }
        $rootScope.beginPageLoad = function(){
            $scope.pageLoading = true;
        }
        $scope.$on('$routeChangeStart', function() {
             $scope.beginPageLoad();
        });

    }])
    .controller('AboutCtrl', ['$scope', '$http', 'deferred', function ($scope, $http, deferred) {
        $scope.$root.currentPage = 1;
        $scope.$root.pageDepth = 1;
        $scope.about = get('About') || [];
        if($scope.about.length){
            $scope.$root.resolvePageLoad();
        }
        $http.get( services.about )
            .success(function(data){
                data.posts = filterAbout(data.posts);
                $scope.about = data;
                set('About', data);
                deferred.resolve();
                $scope.$root.resolvePageLoad();
            });
        $scope.fixContent = function(){
            setTimeout(function(){
            $("a.phone").attr('href', function(){return 'tel:' + this.innerText});
            }, 20);
        }
        setIscroll();
    }])
    .controller('AboutDetailCtrl', ['$scope', '$routeParams', 'deferred', function ($scope, $routeParams, deferred) {
        deferred.resolve();
        $scope.$root.currentPage = 1;
        $scope.$root.pageDepth = 2;
        var about = get('About') || [];
        $scope.$root.backBtnShow = false;
        $scope.$root.backBtn = 'hidden';
        $scope.article = findById(about.posts, $routeParams.articleId);
        $scope.link = jQuery('<div>' + $scope.article.content + '</div>').find('p:last a').attr('href');
        $scope.fixContent = function(){
            setTimeout(function(){
                $("#about-article-content")[0].innerHTML = replaceHref($("#about-article-content")[0].innerHTML);
            }, 20);
        }
        $scope.$root.resolvePageLoad();
   }])
    .controller('CoursesCtrl', ['$scope', '$http', 'deferred', function ($scope, $http, deferred) {
        $scope.$root.beginPageLoad();
        $scope.$root.currentPage = 2;
        $scope.$root.pageDepth = 1;
        var podcastIds = get('podcastIds');
        $scope.courses = get('Courses') || [];
        if($scope.courses.length){
            $scope.prevCourses = $scope.courses;
            $scope.$root.resolvePageLoad();
            deferred.resolve();
        }
        else{
            $scope.prevCourses = false;
        }
        $scope.fetchCourses = function(){
            if(!podcastIds) return;
            $http.get( services.courses, { params: {id: podcastIds}} )
                .success(function(data){
                    //We need to cache the courses here instead of just overwriting them.
                    $scope.courses = data.results;
                    if($scope.prevCourses){
                        for(var i = 0; i < $scope.courses.length; i++){
                            var course = $scope.courses[i],
                                prevCourse = findById($scope.prevCourses, course.trackId, 'collectionId');
                            if(prevCourse){
                                course.feed = prevCourse.feed;
                            }
                        }

                    }
                    set('Courses', data.results);
                    $scope.fetchCourseFeeds();
                    deferred.resolve();
                    $scope.$root.resolvePageLoad();
              });
        }
        $scope.fetchCourseFeeds = function(){
            var courses = get('Courses') || [];
            for(var i = 0; i < courses.length; i++){
                var course = courses[i];

                (function(course){
                    var feedUrl = course.feedUrl;
                    $http.get( DEV ? services.podcast : feedUrl)
                        .success(function(data){
                            var feed = new JFeed(stripCommentsFromXML(data));
                            var courses = get('Courses');
                            course = findById(courses, course.trackId, 'collectionId');
                            course.feed = feed;
                            courses[course.index] = course;
                            $scope.courses = courses;
                            set('Courses', courses);
                        });  
                })(course)
           
            }
        }

        //Get the podcast ids from the about object (parse when you get back from AboutCtrl?)
        //Loop through podcast ids and query https://itunes.apple.com/lookup?id=730083644,730148920,293336438,712006250,304563919,268281421
        $scope.fetchCourses();

    }])
    .controller('CoursesDetailCtrl', ['$scope', '$routeParams', '$http', 'FeedService', '$sce', '$timeout', 'deferred',  function ($scope, $routeParams, $http, Feed, $sce, $timeout, deferred) {
        deferred.resolve();
        var speedSettings = [1, 1.5, 2, 3];
        $scope.$root.currentPage = 2;
        $scope.$root.pageDepth = 2;
        //$scope.speedVisible = false;
        var courses = get('Courses'),
            course = findById(courses, $routeParams.courseId, 'collectionId'),
            feedUrl = course.feedUrl;
        if(course.feed){
            $scope.course = course.feed;
            $scope.$root.resolvePageLoad();
         }
        $scope.$on("$locationChangeStart", function(event) {
            if($scope.currentAudioEntry && $scope.currentAudioEntry.play){
                //Move the audio that is playing to root scope. Allow it to continue playing.
                var $audioElement = $('.audio-playing');
                $audioElement.appendTo($('body'));
                $audioElement.addClass('audio-root');
                if($audioElement[0]){
                    $audioElement[0].play();                    
                }
            }
        });
        $scope.showTimeControl = (!window.device || window.device.version > "2");
        $scope.getAudioElement = function(entry){
            return document.getElementById('audio_entry_' + entry.$$hashKey);
        }
        $scope.getTimeControl = function(entry){
            return document.getElementById('audio_time_' + entry.$$hashKey);
        }
        $scope.changeSpeed = function(entry){
            if(!$scope.currentAudioEntry == entry){
                //If this entry is not currently playing, then play it
                $scope.play(entry);
            }
            else{
                var currentSpeedIndex = speedSettings.indexOf($scope.speed),
                    newSpeedIndex = speedSettings[currentSpeedIndex +1] ? currentSpeedIndex + 1 : 0;
                $scope.speed = speedSettings[newSpeedIndex];
                $scope.audio.play();
                $scope.audio.playbackRate = $scope.speed;
            }
        }
        $scope.showSpeed = function(entry){
            if(!$scope.currentAudioEntry || !$scope.currentAudioEntry == entry){
                //If this entry is not currently playing, then play it
                $scope.play(entry);
            }
            else{
                $scope.currentAudioEntry.speedVisible = true;
                $scope.toggleSpeedVisibleTimeout();
            }
        }
        $scope.toggleSpeedVisibleTimeout = function(){
            $scope.speedTimeout && $timeout.cancel($scope.speedTimeout);
            $scope.speedTimeout = $timeout(function() {
               $scope.currentAudioEntry.speedVisible = !$scope.currentAudioEntry.speedVisible;
               $scope.toggleSpeedVisibleTimeout();
            }, 4000);
        }
        $scope.delayPlay = function(){
            setTimeout(function(){
                $scope.audio.play();
            }, 10);
        };
        $scope.play = function(entry){
            $scope.currentTime = 0;
            if($scope.currentAudioEntry == entry){
                if($scope.audio.paused){
                    $('.audio-root').remove();
                    $scope.delayPlay();
                    entry.play = true;
                }
                else{
                    $scope.audio.pause();
                    entry.play = false;
                }
                return;
            }
            else{
                if($scope.currentAudioEntry){
                    $scope.currentAudioEntry.play = false;
                }
            }
            $scope.toggleSpeedVisibleTimeout();
            $scope.audio && $scope.audio.pause();
            //switch between duration and speed
            //set initial speed
            $scope.speed = speedSettings[0];

            //Remove the global audio element that is playing in the background.
            $('.audio-root').remove();
            $scope.currentAudioEntry = entry;
            $scope.currentAudioEntry.speedVisible = true;
            entry.play = true;
            $scope.audio = $scope.getAudioElement(entry);
            $scope.timeControl = $scope.getTimeControl(entry);
            $scope.audio.addEventListener('timeupdate', function(e){ 
                if($scope.course){
                    for(var i = 0, l = $scope.course.feed.items.length; i < l; i++){
                        var e = $scope.course.feed.items[i];
                        if(e.$$hashKey == entry.$$hashKey){
                            e.elapsedPercent = this.currentTime / this.duration;
                            e.duration = floatToTime(this.duration - this.currentTime);
                            $scope.$apply();
                            $scope.currentTime = this.currentTime;
                            $scope.setEntry(e);
                           return;
                        }
                    }                
                }
            });
            $scope.delayPlay();
        }
        $scope.setEntry = function(entry){
            var courses = get('Courses');
            var course = findById(courses, $routeParams.courseId, 'collectionId');
            if(course && course.feed){
                for(var i = 0, l = course.feed.items.length; i < l; i++){
                    var e = $scope.course.feed.items[i];
                    if(e.$$hashKey == entry.$$hashKey){
                        entry.index = i;
                        course.feed.items[i] = entry;
                        courses[course.index] = course;
                        set('Courses', courses);
                       return;
                    }
                }                
            }
        }
        $scope.getTime = function(entry, t){
            entry.duration = $scope.getAudioElement(entry).duration;
        }
        /*if(courseDetails[$routeParams.courseId]){
            $scope.course = courseDetails[$routeParams.courseId];
        }*/
        $http.get( DEV ? services.podcast : feedUrl)
            .success(function(data){
                var feed = new JFeed(data);
                var courses = get('Courses') || [];
                course = findById(courses, $routeParams.courseId, 'collectionId') || {};
                course.feed = feed;
                courses[course.index || 0] = course;
                set('Courses', courses);
                $scope.course = course;
                $scope.$root.resolvePageLoad();
            });
        $scope.trustAsResourceUrl = function(url){
            return $sce.trustAsResourceUrl(url);
        }
           
    }])
    .controller('BlogCtrl', ['$scope', '$http', 'deferred', '$timeout', function ($scope, $http, deferred, $timeout) {
        $scope.$root.beginPageLoad();
        $scope.$root.currentPage = 3;
        $scope.$root.pageDepth = 1;
        $scope.blog = get('Blog') || {};
        if($scope.blog.items && $scope.blog.items.length){
            deferred.resolve();
            $scope.$root.resolvePageLoad();
            setBlogMasonry();
        }
        $http.get( services.blog )
            .success(function(data){
                $scope.blog = data;
                set('Blog', data);
                $scope.$root.resolvePageLoad();
            });
        $scope.fetchMoreArticles = function(){
            if(window.fetching){
                return true;
            }
            else{
                window.fetching = true;
                $http.get( services.blog, { params: {pageToken: $scope.blog.nextPageToken}} )
                    .success(function(data){
                       deferred.resolve();
                       $scope.blog.items = $scope.blog.items.concat(data.items);
                        $scope.blog.nextPageToken = data.nextPageToken;
                        set('Blog', $scope.blog);
                        $scope.$root.resolvePageLoad();
                        setBlogMasonry();
                        //window.msnry.appended(getLastBlogBricks());
                    });

            }
        }
        //Add a query property "pageToken" equal to data.nextPageToken
    }])
    .controller('BlogDetailCtrl', ['$scope', '$routeParams', 'deferred', function ($scope, $routeParams, deferred) {
        deferred.resolve();
        $scope.$root.currentPage = 3;
        $scope.$root.pageDepth = 2;
        var blog = get('Blog') || {};
        $scope['article'] = findById(blog.items, $routeParams.blogId);
        $scope.shareBlog = function(){
            //To Do: add phone gap share button from https://github.com/EddyVerbruggen/SocialSharing-PhoneGap-Plugin#3-installation
            var message = [
                $scope.article.title + "\n\r",
                $scope.article.url
            ].join('');
            var title = "";
            var link = $scope.article.url;
            share(message, title, link);
        }
        $scope.$root.resolvePageLoad();
        $scope.fixHeight = function(){
            var navHeight = $("#nav").outerHeight();
            var shareBtnContainerHeight = $(".blog-ext-link-container").outerHeight();
            var bannerHeight = $('#banner').outerHeight();
            var windowHeight = $(window).outerHeight();
            var descriptionPadding = $(".blog-details").outerHeight() - $(".blog-details").height();
            //$scope.mapHeight = windowHeight - (navHeight + shareBtnContainerHeight + bannerHeight );
            $scope.descriptionHeight = windowHeight - (navHeight + shareBtnContainerHeight + bannerHeight) - descriptionPadding;
            $scope.buttonPositionTop = Math.floor(bottomLinkHeight());
        };
        $scope.fixHeight();

    }])
    .controller('EventsCtrl', ['$scope', '$http', '$timeout', 'deferred', '$filter', function ($scope, $http, $timeout, deferred, $filter) {
        $scope.$root.beginPageLoad();
        var d = new Date();
        var curr_date = d.getDate();
        var curr_month = ( d.getMonth() < 9 ? '0' : '' ) + (d.getMonth()+1);
        var curr_year = d.getFullYear();
        
        $scope.dateToday = curr_year + '' + curr_month + "" + curr_date;

        $scope.$root.currentPage = 4;
        $scope.$root.pageDepth = 1;
        $scope.events = get('Events') || [];
        if($scope.events && $scope.events.length){
            deferred.resolve();
        }
        $http.get( services.events )
            .success(function(data){
                icalParser.parseIcal(data);
                $scope.events = orderEventsByDate(icalParser.ical.events);
                icalParser.ical.events = [];
                set('Events', $scope.events);
                deferred.resolve();
                $scope.$root.resolvePageLoad();
            });
         $scope.fixTicketWidth = function(){
            var ticketRatio = 224 / 590,
                windowWidth = $(window).width(),
                //This number may change based on the font we use
                descriptionTextHeight = 18,
                margins = 5 + 15;
            //margins += 100//Delete - for testing
            $scope.ticketWidth = windowWidth - margins;
            $scope.ticketHeight = ticketRatio * $scope.ticketWidth;
            $scope.middleContentWidth = ($scope.ticketWidth * .75) - (windowWidth > 420 ? 120 : 90);
            //Make sure that Description Height fits its text
            
            $scope.descriptionHeight = Math.floor(($scope.ticketHeight - 60) / descriptionTextHeight) * descriptionTextHeight;
            $scope.$root.resolvePageLoad();
        }
        $scope.fixTicketWidth();
        $scope.goToCalendar = function( e ){
            goToCalendar(e, $filter);
        }
        $scope.shareEvent = function( e ){
            shareEvent(e, $filter);
        }
   }])
    .controller('EventsDetailCtrl', ['$scope', '$routeParams', 'deferred', '$filter', function ($scope, $routeParams, deferred, $filter) {
        deferred.resolve();
        $scope.$root.currentPage = 4;
        $scope.$root.pageDepth = 2;
        var events = get('Events') || [];
        $scope['event'] = findById(events, $routeParams.eventId, 'uid.value');
         $scope.shareEvent = function( e ){
            shareEvent(e, $filter);
        }
        $scope.goToCalendar = function( e ){
            goToCalendar(e, $filter);
        }
        $scope.trustAsResourceUrl = function(url){
            return $sce.trustAsResourceUrl(url);
        }
        $scope.openMap = function(event){
            //TO Do: 
        }
        $scope.fixHeight = function(){
            var navHeight = $("#nav").outerHeight();
            var shareBtnContainerHeight = $(".event-details-ext-link-container").outerHeight();
            var eventDeatilsMidHeight = $(".event-details-mid").outerHeight();
            var imgheaderHeight = $(".img-header").outerHeight();
            var bannerHeight = $('#banner').outerHeight();
            var windowHeight = $(window).outerHeight();
            $scope.buttonPositionTop = Math.floor(bottomLinkHeight());
            //$scope.mapHeight = windowHeight - (navHeight + shareBtnContainerHeight + bannerHeight );
            $scope.mapHeight = windowHeight - (navHeight + shareBtnContainerHeight + bannerHeight );
            $scope.mapBottom = navHeight;
            $scope.descriptionHeight = windowHeight - (navHeight + shareBtnContainerHeight + bannerHeight + eventDeatilsMidHeight + imgheaderHeight);
        }
        $scope.toggleMap = function(){
            $scope.mapVisible = !$scope.mapVisible;
        }
        $scope.loadMap = function(){
            var location = $scope['event'].location.value.split(',').slice(1).join(', ').replace(/\\,/g, '');
            if(!get('googleLocations')){
                set('googleLocations', {});
            }
            var googleLocations = get('googleLocations');
            if(!googleLocations[location]){
                var geocoder = new google.maps.Geocoder();
                geocoder.geocode( { 'address': location}, function(results, status) {
                    googleLocations[location] = {
                        lat: results[0].geometry.location.lat(),
                        long: results[0].geometry.location.lng()
                    }
                    $scope.loadMap();
                })
            }
            else{
                var map = new GoogleMap(googleLocations[location], location);
                map.initialize();
                GoogleMap();
            }

        }
        $scope.fixHeight();
        $scope.$root.resolvePageLoad();
   }])
    